@@ -242,18 +242,21 @@ int identical_ban(const char *one, const char *two)
 /** Add a listmode (+beI) with the specified banid to
  *  the specified channel. (Extended version with
  *  set by nick and set on timestamp)
+ * @retval	1	Added
+ * @retval	0	Updated
+ * @retval	-1	Ban list full
  */
 int add_listmode_ex(Ban **list, Client *client, Channel *channel, const char *banid, const char *setby, time_t seton)
 {
 	Ban *ban;
-	int cnt = 0, len;
+	int cnt = 0;
 	int do_not_add = 0;
+	char isnew = 0;
 
 	//if (MyUser(client))
 	//	collapse(banid);
 
-	len = strlen(banid);
-	if (!*list && ((len > MAXBANLENGTH) || (MAXBANS < 1)))
+	if (!*list && (MAXBANS < 1))
 	{
 		if (MyUser(client))
 		{
@@ -264,14 +267,10 @@ int add_listmode_ex(Ban **list, Client *client, Channel *channel, const char *ba
 	}
 	for (ban = *list; ban; ban = ban->next)
 	{
-		len += strlen(ban->banstr);
-		/* Check MAXBANLENGTH / MAXBANS only for local clients
-		 * and 'me' (for +b's set during +f).
+		/* Check MAXBANS only for local clients and 'me' (for +b's set during +f).
 		 */
-		if ((MyUser(client) || IsMe(client)) && ((len > MAXBANLENGTH) || (++cnt >= MAXBANS)))
-		{
+		if ((MyUser(client) || IsMe(client)) && (++cnt >= MAXBANS))
 			do_not_add = 1;
-		}
 		if (identical_ban(ban->banstr, banid))
 			break; /* update existing ban (potentially) */
 	}
@@ -294,6 +293,7 @@ int add_listmode_ex(Ban **list, Client *client, Channel *channel, const char *ba
 		ban = make_ban();
 		ban->next = *list;
 		*list = ban;
+		isnew = 1;
 	}
 
 	if ((ban->when > 0) && (seton >= ban->when))
@@ -308,7 +308,7 @@ int add_listmode_ex(Ban **list, Client *client, Channel *channel, const char *ba
 	safe_strdup(ban->banstr, banid); /* cAsE may differ, use oldest version of it */
 	safe_strdup(ban->who, setby);
 	ban->when = seton;
-	return 0;
+	return isnew ? 1 : 0;
 }
 
 /** Add a listmode (+beI) with the specified banid to
@@ -367,7 +367,6 @@ inline Ban *is_banned(Client *client, Channel *channel, int type, const char **m
 /** ban_check_mask - Checks if the user matches the specified n!u@h mask -or- run an extended ban.
  * This is basically extracting the mask and extban check from is_banned_with_nick,
  * but with being a bit more strict in what an extban is.
- * Strange things could happen if this is called outside standard ban checking.
  * @param b	Ban context, see BanContext
  * @returns	Nonzero if the mask/extban succeeds. Zero if it doesn't.
  */
@@ -434,6 +433,7 @@ Ban *is_banned_with_nick(Client *client, Channel *channel, int type, const char
 	 * If a +e was found we return NULL, if not, we return the ban.
 	 */
 
+	b->ban_type = EXBTYPE_BAN;
 	for (ban = channel->banlist; ban; ban = ban->next)
 	{
 		b->banstr = ban->banstr;
@@ -444,6 +444,7 @@ Ban *is_banned_with_nick(Client *client, Channel *channel, int type, const char
 	if (ban)
 	{
 		/* Ban found, now check for +e */
+		b->ban_type = EXBTYPE_EXCEPT;
 		for (ex = channel->exlist; ex; ex = ex->next)
 		{
 			b->banstr = ex->banstr;
@@ -473,6 +474,46 @@ Ban *is_banned_with_nick(Client *client, Channel *channel, int type, const char
 	return ban;
 }
 
+/** Checks if a ban already exists */
+int ban_exists(Ban *lst, const char *str)
+{
+	for (; lst; lst = lst->next)
+		if (!mycmp(lst->banstr, str))
+			return 1;
+	return 0;
+}
+
+/** Checks if a ban already exists - special version.
+ * This ignores the "~time:xx:" suffixes in the banlist.
+ * So it will return 1 if a ban is there for ~time:5:blah!*@*
+ * and you call ban_exists_ignore_time(channel->banlist, "blah!*@*")
+ */
+int ban_exists_ignore_time(Ban *lst, const char *str)
+{
+	const char *p;
+
+	for (; lst; lst = lst->next)
+	{
+		if (!strncmp(lst->banstr, "~time:", 6))
+		{
+			/* Special treatment for ~time:xx: */
+			p = strchr(lst->banstr+6, ':');
+			if (p)
+			{
+				p++;
+				if (!mycmp(p, str))
+					return 1;
+			}
+		} else
+		{
+			/* The simple version */
+			if (!mycmp(lst->banstr, str))
+				return 1;
+		}
+	}
+	return 0;
+}
+
 /** Add user to the channel.
  * This adds both the Member struct to the channel->members linked list
  * and also the Membership struct to the client->user->channel linked list.
@@ -751,15 +792,17 @@ const char *convert_regular_ban(char *mask, char *buf, size_t buflen)
  * This takes user input (eg: "nick") and converts it to a mask suitable
  * in the +beI lists (eg: "nick!*@*"). It also deals with extended bans,
  * in which case it will call the extban->conv_param() function.
- * @param mask		The ban mask
+ * @param mask_in	The ban mask
  * @param what		MODE_DEL or MODE_ADD
+ * @param ban_type	One of EXBTYPE_*, such as EXBTYPE_BAN.
  * @param client	The client adding/removing this ban mask
+ * @param channel	The channel on which this entry will be added or removed
  * @param conv_options	Options for BanContext.conv_options (eg BCTX_CONV_OPTION_WRITE_LETTER_BANS)
  * @returns pointer to correct banmask or NULL in case of error
  * @note A pointer is returned to a static buffer, which is overwritten
  *       on next clean_ban_mask or make_nick_user_host call.
  */
-const char *clean_ban_mask(const char *mask_in, int what, Client *client, int conv_options)
+const char *clean_ban_mask(const char *mask_in, int what, ExtbanType ban_type, Client *client, Channel *channel, int conv_options)
 {
 	char *cp, *x;
 	static char mask[512];
@@ -823,7 +866,9 @@ const char *clean_ban_mask(const char *mask_in, int what, Client *client, int co
 			static char retbuf[512];
 			BanContext *b = safe_alloc(sizeof(BanContext));
 			b->client = client;
+			b->channel = channel;
 			b->what = what;
+			b->ban_type = ban_type;
 			b->banstr = nextbanstr;
 			b->conv_options = conv_options;
 			ret = extban->conv_param(b, extban);
@@ -851,6 +896,7 @@ int find_invex(Channel *channel, Client *client)
 	b->client = client;
 	b->channel = channel;
 	b->ban_check_types = BANCHK_JOIN;
+	b->ban_type = EXBTYPE_INVEX;
 
 	for (inv = channel->invexlist; inv; inv = inv->next)
 	{
@@ -1233,11 +1279,15 @@ int has_common_channels(Client *c1, Client *c2)
 	return 0;
 }
 
-/** Returns 1 if user 'user' can see channel member 'target'.
+/** Returns 1 if user 'user' can see channel member 'target' - fast version.
  * This may return 0 if the user is 'invisible' due to mode +D rules.
- * NOTE: Membership is unchecked, assumed membership of both.
+ * @param user			The user who is looking around
+ * @param target		The target user who is being investigated
+ * @param channel		The channel
+ * @param target_member		The Member * struct of 'target'
+ * @param user_member_modes	The member modes that 'user' has, eg "o". Can be NULL if not in channel.
  */
-int user_can_see_member(Client *user, Client *target, Channel *channel)
+int user_can_see_member_fast(Client *user, Client *target, Channel *channel, Member *target_member, const char *user_member_modes)
 {
 	Hook *h;
 	int j = 0;
@@ -1247,34 +1297,69 @@ int user_can_see_member(Client *user, Client *target, Channel *channel)
 
 	for (h = Hooks[HOOKTYPE_VISIBLE_IN_CHANNEL]; h; h = h->next)
 	{
-		j = (*(h->func.intfunc))(target,channel);
+		j = (*(h->func.intfunc))(target, channel, target_member);
 		if (j != 0)
 			break;
 	}
 
-	/* We must ensure that user is allowed to "see" target */
-	if (j != 0 && !(check_channel_access(target, channel, "hoaq") || check_channel_access(target,channel, "v")) && !check_channel_access(user, channel, "hoaq"))
+	/* Requested to hide the person, but make sure neither one is +hoaq... */
+	if ((j != 0) &&
+	    !check_channel_access_member(target_member, "vhoaq") &&
+	    !(user_member_modes && check_channel_access_string(user_member_modes, "hoaq")))
+	{
 		return 0;
+	}
 
 	return 1;
 }
 
+/** Returns 1 if user 'user' can see channel member 'target'.
+ * This may return 0 if the user is 'invisible' due to mode +D rules.
+ * NOTE: Membership is unchecked, assumed membership of both.
+ */
+int user_can_see_member(Client *user, Client *target, Channel *channel)
+{
+	Membership *user_member = NULL;
+	Member *target_member = NULL;
+
+	if (user == target)
+		return 1;
+
+	if (IsUser(user))
+		user_member = find_membership_link(user->user->channel, channel);
+
+	if (IsUser(target))
+		target_member = find_member_link(channel->members, target); // SLOW!
+
+	/* User is not in channel, yeah what shall we return? :D */
+	if (!target_member)
+		return 0;
+
+	return user_can_see_member_fast(user, target, channel, target_member, user_member ? user_member->member_modes : NULL);
+}
+
 /** Returns 1 if user 'target' is invisible in channel 'channel'.
  * This may return 0 if the user is 'invisible' due to mode +D rules.
  */
 int invisible_user_in_channel(Client *target, Channel *channel)
 {
 	Hook *h;
+	Member *target_member;
 	int j = 0;
 
+	target_member = find_member_link(channel->members, target); // SLOW!
+	if (!target_member)
+		return 0; /* not in channel */
+
 	for (h = Hooks[HOOKTYPE_VISIBLE_IN_CHANNEL]; h; h = h->next)
 	{
-		j = (*(h->func.intfunc))(target,channel);
+		j = (*(h->func.intfunc))(target,channel,target_member);
 		if (j != 0)
 			break;
 	}
 
 	/* We must ensure that user is allowed to "see" target */
+	// SLOW !!!
 	if (j != 0 && !(check_channel_access(target, channel, "hoaq") || check_channel_access(target,channel, "v")))
 		return 1;
 
@@ -1374,3 +1459,13 @@ void free_multilinemode(MultiLineMode *m)
 	}
 	safe_free(m);
 }
+
+ExtbanType mode_letter_to_extbantype(char c)
+{
+	if (c == 'e')
+		return EXBTYPE_EXCEPT;
+	else if (c == 'I')
+		return EXBTYPE_INVEX;
+	/* Else default to 'b' */
+	return EXBTYPE_BAN;
+}
