@@ -1293,7 +1293,7 @@ int i;
 	if (!Callbacks[CALLBACKTYPE_CLOAK_KEY_CHECKSUM])
 	{
 		unreal_log(ULOG_ERROR, "config", "NO_CLOAKING_MODULE", NULL,
-		           "No cloaking module loaded, you must load 1 of these modulese:\n"
+		           "No cloaking module loaded, you must load 1 of these modules:\n"
 		           "1) cloak_sha256 - if you are a new network starting with UnrealIRCd 6\n"
 		           "2) cloak_md5 - the old one if migrating an existing network from UnrealIRCd 3.2/4/5\n"
 		           "3) cloak_none - if you don't want to use cloaking at all\n"
