@@ -65,7 +65,6 @@ int stats_port(Client *, const char *);
 int stats_bannick(Client *, const char *);
 int stats_traffic(Client *, const char *);
 int stats_uline(Client *, const char *);
-int stats_vhost(Client *, const char *);
 int stats_kline(Client *, const char *);
 int stats_banrealname(Client *, const char *);
 int stats_sqline(Client *, const char *);
@@ -84,7 +83,6 @@ int stats_officialchannels(Client *, const char *);
 int stats_spamfilter(Client *, const char *);
 int stats_fdtable(Client *, const char *);
 int stats_linecache(Client *client, const char *para);
-int stats_maxperip(Client *, const char *);
 
 #define SERVER_AS_PARA 0x1
 #define FLAGS_AS_PARA 0x2
@@ -113,7 +111,6 @@ struct statstab StatsTable[] = {
 	{ 'S', "set",		stats_set,		0		},
 	{ 'T', "traffic",	stats_traffic,		0 		},
 	{ 'U', "uline",		stats_uline,		0 		},
-	{ 'V', "vhost", 	stats_vhost,		0 		},
 	{ 'W', "fdtable",       stats_fdtable,          0               },
 	{ 'X', "notlink",	stats_notlink,		0 		},
 	{ 'Y', "class",		stats_class,		0 		},
@@ -136,7 +133,6 @@ struct statstab StatsTable[] = {
 	{ 'v', "denyver",	stats_denyver,		0 		},
 	{ 'x', "notlink",	stats_notlink,		0 		},
 	{ 'y', "class",		stats_class,		0 		},
-	{ '8', "maxperip",	stats_maxperip,		0		},
 	{ '9', "linecache",	stats_linecache,	0		},
 	{ 0, 	NULL, 		NULL, 			0		}
 };
@@ -650,25 +646,6 @@ int stats_uline(Client *client, const char *para)
 		sendnumeric(client, RPL_STATSULINE, ulines->servername);
 	return 0;
 }
-int stats_vhost(Client *client, const char *para)
-{
-	ConfigItem_vhost *vhosts;
-	NameValuePrioList *m;
-
-	for (vhosts = conf_vhost; vhosts; vhosts = vhosts->next)
-	{
-		for (m = vhosts->match->printable_list; m; m = m->next)
-		{
-			sendtxtnumeric(client, "vhost %s%s%s %s %s",
-			               vhosts->virtuser ? vhosts->virtuser : "",
-			               vhosts->virtuser ? "@" : "",
-			               vhosts->virthost,
-			               vhosts->login,
-			               namevalue_nospaces(m));
-		}
-	}
-	return 0;
-}
 
 int stats_kline(Client *client, const char *para)
 {
@@ -1081,45 +1058,3 @@ int stats_linecache(Client *client, const char *para)
 
 	return 0;
 }
-
-int stats_maxperip(Client *client, const char *para)
-{
-	int i;
-	IpUsersBucket *e;
-	char ipbuf[256];
-	const char *ip;
-
-	if (!ValidatePermissionsForPath("server:info:stats",client,NULL,NULL,NULL))
-	{
-		sendnumeric(client, ERR_NOPRIVILEGES);
-		return 0;
-	}
-
-	sendtxtnumeric(client, "MaxPerIp IPv4 hash table:");
-	for (i=0; i < IPUSERS_HASH_TABLE_SIZE; i++)
-	{
-		for (e = IpUsersHash_ipv4[i]; e; e = e->next)
-		{
-			ip = inetntop(AF_INET, e->rawip, ipbuf, sizeof(ipbuf));
-			if (!ip)
-				ip = "<invalid>";
-			sendtxtnumeric(client, "IPv4 #%d %s: %d local / %d global",
-				       i, ip, e->local_clients, e->global_clients);
-		}
-	}
-
-	sendtxtnumeric(client, "MaxPerIp IPv6 hash table:");
-	for (i=0; i < IPUSERS_HASH_TABLE_SIZE; i++)
-	{
-		for (e = IpUsersHash_ipv6[i]; e; e = e->next)
-		{
-			ip = inetntop(AF_INET6, e->rawip, ipbuf, sizeof(ipbuf));
-			if (!ip)
-				ip = "<invalid>";
-			sendtxtnumeric(client, "IPv6 #%d %s: %d local / %d global",
-				       i, ip, e->local_clients, e->global_clients);
-		}
-	}
-
-	return 0;
-}
