@@ -26,7 +26,7 @@
 ModuleHeader MOD_HEADER
   = {
 	"vhost",
-	"6.0",
+	"6.1.8.1",
 	"command /VHOST and vhost { } blocks",
 	"UnrealIRCd Team",
 	"unrealircd-6",
@@ -360,10 +360,8 @@ ConfigItem_vhost *find_vhost(const char *name)
 	ConfigItem_vhost *vhost;
 
 	for (vhost = conf_vhost; vhost; vhost = vhost->next)
-	{
-		if (!strcmp(name, vhost->login))
+		if (vhost->login && !strcmp(name, vhost->login))
 			return vhost;
-	}
 
 	return NULL;
 }
