@@ -190,10 +190,9 @@ static int ssl_hostname_callback(SSL *ssl, int *unk, void *arg)
 	ConfigItem_sni *sni;
 
 	if (name && (sni = find_sni(name)))
-	{
 		SSL_set_SSL_CTX(ssl, sni->ssl_ctx);
-		set_client_sni_name(ssl, name);
-	}
+
+	set_client_sni_name(ssl, name);
 
 	return SSL_TLSEXT_ERR_OK;
 }
@@ -918,10 +917,12 @@ static int fatal_tls_error(int ssl_error, int where, int my_errno, Client *clien
 
 	SetDeadSocket(client);
 	unreal_log(ULOG_DEBUG, "tls", "DEBUG_TLS_FATAL_ERROR", client,
-		   "Exiting TLS client $client.details: $tls_function: $tls_error_string: $tls_additional_info",
+		   "Exiting TLS client $client.details [port $port]: $tls_function: $tls_error_string: $tls_additional_info",
 		   log_data_string("tls_function", ssl_func),
 		   log_data_string("tls_error_string", ssl_errstr),
-		   log_data_string("tls_additional_info", additional_info));
+		   log_data_string("tls_additional_info", additional_info),
+		   log_data_integer("port", client->local->listener ? client->local->listener->port : 0));
+
 
 	if (where == FUNC_TLS_CONNECT)
 	{
@@ -1429,6 +1430,9 @@ void check_certificate_expiry_tlsoptions_and_warn(TLSOptions *tlsoptions)
 
 EVENT(tls_check_expiry)
 {
+	if (iConf.tls_options->certificate_expiry_notification == 0)
+		return;
+
 	ConfigItem_listen *listen;
 	ConfigItem_sni *sni;
 	ConfigItem_link *link;
